unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  Menus, LCLType, LazFileUtils, ComCtrls, ActnList, blcksock,
  uTCPSockets, uClient, uSettings;

type

  { TfMain }

  TfMain = class(TForm)
    acConnect: TAction;
    acDisconnect: TAction;
    acQuit: TAction;
    acOptions: TAction;
    acAbout: TAction;
    alMain: TActionList;
    imSB: TImageList;
    lbLog: TListBox;
    MenuItem1: TMenuItem;
    miHideErrorMessages: TMenuItem;
    miHideInternalMessages: TMenuItem;
    miHide: TMenuItem;
    miSave: TMenuItem;
    mm: TMainMenu;
    miAbout: TMenuItem;
    miHelp: TMenuItem;
    miOptions: TMenuItem;
    miTools: TMenuItem;
    miDisconnect: TMenuItem;
    miConnect: TMenuItem;
    miConnection: TMenuItem;
    miQuit: TMenuItem;
    miFile: TMenuItem;
    miClear: TMenuItem;
    pnLog: TPanel;
    pnMain: TPanel;
    pmLog: TPopupMenu;
    pnClient: TPanel;
    SD: TSaveDialog;
    sb: TStatusBar;
    sp: TSplitter;
    tmWait: TTimer;
    procedure acAboutExecute(Sender: TObject);
    procedure acConnectExecute(Sender: TObject);
    procedure acDisconnectExecute(Sender: TObject);
    procedure acQuitExecute(Sender: TObject);
    procedure acOptionsExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var {%H-}CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miClearClick(Sender: TObject);
    procedure miHideErrorMessagesClick(Sender: TObject);
    procedure miHideInternalMessagesClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure pmLogPopup(Sender: TObject);
    procedure sbDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
      const Rect: TRect);
    procedure tmWaitTimer(Sender: TObject);
  private
    FNeedToClose: Boolean;
    FTCPClient: TTCPClient;
    FSettings: TSettings;
    FRClient: TfrClient;
    FConCap: String;
    FDisCap: String;
    procedure AddLog(ALogMsg: String);
    procedure DoOnConnect(Sender: TObject; AConnection: TConnection);
    procedure DoOnDisconnect(Sender: TObject; {%H-}AConnection: TConnection; AByUser: Boolean);
    procedure DoOnError(Sender: TObject; AErrMsg: string; AErrCode: Integer; {%H-}AConnection: TConnection);
    procedure DoOnInternalMessage(Sender: TObject; AMsg: String);
    procedure DoOnProgress(Sender: TObject; AStreamingType: TStreamingType; ACnt, ATotCnt: Int64; ASpeed, {%H-}ARemaining: LongInt; {%H-}AConnection: TConnection);
    procedure DoOnRecvMessage(Sender: TObject; AMsg: String; AParams: TStringArray; AConnection: TConnection);
    procedure DoOnRecvStream(Sender: TObject; AMsg: String; AParams: TStringArray; AMS: TMemoryStream; AConnection: TConnection);
    procedure DoOnRecvFile(Sender: TObject; AMsg: String; AParams: TStringArray; AFileName: String; AConnection: TConnection);
  public

  end;

const
  Version = '1.0';
  SynTrunkRev = 209;

var
  fMain: TfMain;

implementation

uses uLogin, uOptions, uAbout, uTCPFunctions, uCommon;
{$R *.lfm}

{ TfMain }

procedure TfMain.FormCreate(Sender: TObject);
begin
  FTCPClient := TTCPClient.Create;
  FTCPClient.OnConnect := @DoOnConnect;
  FTCPClient.OnDisconnect := @DoOnDisconnect;
  FTCPClient.OnError := @DoOnError;
  FTCPClient.OnInternalMessage := @DoOnInternalMessage;
  FTCPClient.OnProgress := @DoOnProgress;
  FTCPClient.OnRecvMessage := @DoOnRecvMessage;
  FTCPClient.OnRecvStream := @DoOnRecvStream;
  FTCPClient.OnRecvFile := @DoOnRecvFile;

  FSettings := TSettings.Create;
  FSettings.FileName := AppendPathDelim(ExtractFilePath(Application.Exename)) + 'conf.xml';

  sb.Tag := 0;
  sb.Panels[0].Text := 'Idle. Waiting for commands...';

  FRClient := TfrClient.Create(Self);
  FRClient.Parent := pnClient;
  FRClient.Align := alClient;
  FRClient.InitFrame(FTCPClient, btClient, FSettings, FSettings.ActiveConnection);
  FRClient.pgPages.Visible := False;
  FRClient.tvPages.Visible := False;
  FDisCap := 'Client - not connected';
  FConCap := '';
  Caption := FDisCap;
end;

procedure TfMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  FNeedToClose := True;
  if FTCPClient.Connected then
    FTCPClient.Disconnect(False);
end;

procedure TfMain.FormDestroy(Sender: TObject);
begin
  FRClient.Free;
  FSettings.Free;
  FTCPClient.Free;
end;

procedure TfMain.FormShow(Sender: TObject);
begin
  tmWait.Enabled := True;
end;

procedure TfMain.tmWaitTimer(Sender: TObject);
begin
  tmWait.Enabled := False;
  acConnectExecute(acConnect);
end;

procedure TfMain.miClearClick(Sender: TObject);
begin
  lbLog.Clear;
end;

procedure TfMain.miHideInternalMessagesClick(Sender: TObject);
begin
  miHideInternalMessages.Checked := not miHideInternalMessages.Checked;
  FSettings.HideInternalMessages := miHideInternalMessages.Checked;
  FTCPClient.HideInternalMessages := miHideInternalMessages.Checked;
  FSettings.Save;
end;

procedure TfMain.miHideErrorMessagesClick(Sender: TObject);
begin
  miHideErrorMessages.Checked := not miHideErrorMessages.Checked;
  FSettings.HideErrorMessages := miHideErrorMessages.Checked;
  FTCPClient.HideErrorMessages := miHideErrorMessages.Checked;
  FSettings.Save;
end;

procedure TfMain.pmLogPopup(Sender: TObject);
begin
  FSettings.Load;
  miHideInternalMessages.Checked := FSettings.HideInternalMessages;
  miHideErrorMessages.Checked := FSettings.HideErrorMessages;
end;


procedure TfMain.miSaveClick(Sender: TObject);
begin
  SD.FileName := 'Log';
  if SD.Execute then
    lbLog.Items.SaveToFile(SD.FileName);
end;


procedure TfMain.sbDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
  const Rect: TRect);
var
  H, T, L: Integer;
begin
  with StatusBar.Canvas do
  begin
    H := StatusBar.Canvas.GetTextHeight('test');
    T := Rect.Top + (Rect.Bottom - Rect.Top - H) div 2;
    L := Rect.Left + 5;
    case Panel.Index of
      0: begin
           case sb.Tag of
             0: imSB.Draw(StatusBar.Canvas, L, T, 0);
             1: imSB.Draw(StatusBar.Canvas, L, T, 1);
             2: imSB.Draw(StatusBar.Canvas, L, T, 2);
           end;
           TextRect(Rect, L + ImSb.Width + 5, T, Panel.Text);
        end;
    end;
  end;
end;

procedure TfMain.AddLog(ALogMsg: String);
begin
  lbLog.Items.Add(FormatDateTime('hh:mm:ss', now) + '  ' + ALogMsg);
  lbLog.TopIndex := lbLog.Items.Count - 1;
end;

procedure TfMain.acQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfMain.acOptionsExecute(Sender: TObject);
begin
  fOptions := TfOptions.Create(nil, FSettings);
  try
    fOptions.ShowModal;
    FTCPClient.DownloadDirectory := FSettings.DownloadDirectory;
    FTCPClient.HideInternalMessages := FSettings.HideInternalMessages;
    FTCPClient.HideErrorMessages := FSettings.HideErrorMessages;
  finally
    fOptions.Free;
  end;
end;

procedure TfMain.acConnectExecute(Sender: TObject);
begin
  fLogin := TfLogin.Create(nil, FTCPClient, FSettings);
  try
    pnMain.Visible := False;
    fLogin.ShowModal;
    pnMain.Visible := True;
  finally
    fLogin.Free;
  end;
  FTCPClient.OnError := @DoOnError;
end;

procedure TfMain.acDisconnectExecute(Sender: TObject);
begin
  acDisconnect.Enabled := False;
  acConnect.Enabled := True;
  FTCPClient.Disconnect(True);
end;

procedure TfMain.acAboutExecute(Sender: TObject);
begin
  fAbout := TfAbout.Create(nil);
  try
    fAbout.ShowModal;
  finally
    fAbout.Free;
  end;
end;

procedure TfMain.DoOnConnect(Sender: TObject; AConnection: TConnection);
var
  ProxyType: String;
  ProxyStr: String;
  ConStr: String;
begin
  if AConnection.FProxy.FProxyType <> ptNone then
  begin
    case AConnection.FProxy.FProxyType  of
      ptHTML: ProxyType := 'HTML';
      ptSocks4: ProxyType := 'Socks4';
      ptSocks5: ProxyType := 'Socks5';
      else
        ProxyType := 'unknown';
    end;
    ProxyStr := '(via ' + AConnection.FProxy.FProxyIP + ' ' + ProxyType + ' proxy server).';
    ConStr := 'Connected to: ' + AConnection.FExtendedInfo.FRemoteIP + ProxyStr;
  end
  else
    ConStr := 'Connected to: ' + AConnection.FIP + '.';
  AddLog(ConStr);
  sb.Tag := 1;
  sb.Panels[0].Text := ConStr;
  acConnect.Enabled := False;
  acDisconnect.Enabled := True;
  FRClient.pgPages.Visible := True;
  FRClient.tvPages.Visible := True;
  FConCap := 'Client - connected to' + AConnection.FExtendedInfo.FRemoteIP + '(' + AConnection.FExtendedInfo.FRemoteHost + '), as user "' + AConnection.FUser + '"';
  Caption := FConCap;
end;

procedure TfMain.DoOnDisconnect(Sender: TObject; AConnection: TConnection;
  AByUser: Boolean);
var
  DisStr: String;
begin
  if AByUser then
    DisStr := 'Disconnected by user.'
  else
    DisStr := 'Disconnected.';
  if not FNeedToClose then
  begin
    AddLog(DisStr);
    sb.Tag := 2;
    sb.Panels[0].Text := DisStr;
    acConnect.Enabled := True;
    acDisconnect.Enabled := False;
    pnMain.Visible := True;
    FRClient.pgPages.Visible := False;
    FRClient.tvPages.Visible := False;
    if not AByUser then
      if MessageDlgEx('Connection lost. Do you wish to (re)conect?', mtConfirmation, [mbYes, mbNo], [], False, Self) = mrYes then
        tmWait.Enabled := True;
    Caption := FDisCap;
  end;
end;

procedure TfMain.DoOnError(Sender: TObject; AErrMsg: string; AErrCode: Integer;
  AConnection: TConnection);
begin
  AddLog(AErrMsg + '(' + IntToStr(AErrCode) + ').');
end;

procedure TfMain.DoOnInternalMessage(Sender: TObject; AMsg: String);
begin
  AddLog(AMsg);
end;

procedure TfMain.DoOnProgress(Sender: TObject; AStreamingType: TStreamingType;
  ACnt, ATotCnt: Int64; ASpeed, ARemaining: LongInt; AConnection: TConnection);
var
  SpeedStr: String;
begin
  if ASpeed > 0 then
    SpeedStr := ', speed: ' + FormatSpeed(ASpeed) + ', remaining: ' + SecToHourAndMin(ARemaining)
  else
    SpeedStr := '';
  case AStreamingType of
    stUpload: Caption := 'Sent ' + FormatSize(ACnt) + '/' + FormatSize(ATotCnt) + SpeedStr;
    stDownload: Caption := 'Received ' + FormatSize(ACnt) + '/' + FormatSize(ATotCnt) + SpeedStr;
  end;
  if ACnt >= ATotCnt then
    Caption := FConCap;
  Application.ProcessMessages;
end;

procedure TfMain.DoOnRecvMessage(Sender: TObject; AMsg: String;
  AParams: TStringArray; AConnection: TConnection);
type
  TAddLog = procedure(ALogMsg: String) of Object;
var
  AddLogCallback: TAddLog;
begin
  if FRClient <> nil then
  begin
    AddLogCallback := @AddLog;
    FRClient.RecvMessage(Sender, AMsg, AParams, AConnection, AddLogCallBack);
  end;
end;

procedure TfMain.DoOnRecvStream(Sender: TObject; AMsg: String;
  AParams: TStringArray; AMS: TMemoryStream; AConnection: TConnection);
type
  TAddLog = procedure(ALogMsg: String) of Object;
var
  AddLogCallback: TAddLog;
begin
  if FRClient <> nil then
  begin
    AddLogCallback := @AddLog;
    FRClient.RecvStream(Sender, AMsg, AParams, AMS, AConnection, AddLogCallback);
  end;
end;

procedure TfMain.DoOnRecvFile(Sender: TObject; AMsg: String;
  AParams: TStringArray; AFileName: String; AConnection: TConnection);
type
  TAddLog = procedure(ALogMsg: String) of Object;
var
  AddLogCallback: TAddLog;
begin
  if FRClient <> nil then
  begin
    AddLogCallback := @AddLog;
    FRClient.RecvFile(Sender, AMsg, AParams, AFileName, AConnection, AddLogCallback);
  end;
end;

end.

