unit uOptions;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls,
  ExtCtrls, EditBtn, LazFileUtils, uSettings;

type

  { TfOptions }

  TfOptions = class(TForm)
    bCancel: TButton;
    bOk: TButton;
    bDefaults: TButton;
    cbAutoConnect: TCheckBox;
    cbConnection: TCheckBox;
    cbCredentials: TCheckBox;
    cbErrorMessages: TCheckBox;
    cbInternalMessages: TCheckBox;
    deDownloads: TDirectoryEdit;
    lbDownload: TLabel;
    pgOptions: TPageControl;
    pnBottom: TPanel;
    tsFiles: TTabSheet;
    tsMessages: TTabSheet;
    tsGeneral: TTabSheet;
    procedure bDefaultsClick(Sender: TObject);
    procedure bOkClick(Sender: TObject);
    procedure cbConnectionClick(Sender: TObject);
  private
    FSettings: TSettings;
    procedure LoadSettings;
    procedure SaveSettings;
  public
    constructor Create(TheOwner: TComponent; ASettings: TSettings); reintroduce;
  end;

var
  fOptions: TfOptions;

implementation

uses uCommon;

{$R *.lfm}

{ TfOptions }

constructor TfOptions.Create(TheOwner: TComponent; ASettings: TSettings);
begin
  inherited Create(TheOwner);
  FSettings := ASettings;
  LoadSettings;
  pgOptions.ActivePageIndex := 0;
end;

procedure TfOptions.bDefaultsClick(Sender: TObject);
begin
  if MessageDlgEx('Restore default settings?', mtConfirmation, [mbYes, mbNo], [], False, Self) = mrYes then
  begin
    cbConnection.Checked := True;
    cbCredentials.Checked := False;
    cbAutoConnect.Checked := False;
    cbInternalMessages.Checked := False;
    cbErrorMessages.Checked := False;
    deDownloads.Directory := AppendPathDelim(GetUserDir) + 'Downloads';
  end;
end;

procedure TfOptions.bOkClick(Sender: TObject);
begin
  SaveSettings;
  ModalResult := mrOk;
end;

procedure TfOptions.LoadSettings;
begin
  with FSettings do
  begin
    Load;
    cbConnection.Checked := SaveConnections;
    cbCredentials.Enabled := SaveConnections;
    cbCredentials.Checked := SaveCredentials;
    cbAutoConnect.Enabled := SaveConnections;
    cbAutoConnect.Checked := AutoConnect;
    cbInternalMessages.Checked := HideInternalMessages;
    cbErrorMessages.Checked := HideErrorMessages;
    deDownloads.Directory := DownloadDirectory;
  end;
end;

procedure TfOptions.SaveSettings;
begin
  with FSettings do
  begin
    AutoConnect := cbAutoConnect.Checked;
    SaveConnections := cbConnection.Checked;
    SaveCredentials := cbCredentials.Checked;
    HideInternalMessages := cbInternalMessages.Checked;
    HideErrorMessages := cbErrorMessages.Checked;
    DownloadDirectory := deDownloads.Directory;
    Save;
  end;
end;

procedure TfOptions.cbConnectionClick(Sender: TObject);
begin
  cbCredentials.Enabled := cbConnection.Checked;
  cbAutoConnect.Enabled := cbConnection.Checked;
  if not cbConnection.Checked then
  begin
    cbCredentials.Checked := False;
    cbAutoConnect.Checked := False;
  end;
end;


end.

