unit uClient;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Dialogs, Graphics, ExtCtrls, ComCtrls,
  StdCtrls, Menus, ExtDlgs, ShellCtrls, EditBtn, LazFileUtils, uTCPSockets,
  uSettings;

type

  { TfrClient }
  TAddLog = procedure(ALogMsg: String) of Object;
  TfrClient = class(TFrame)
    bClearBroadcastedFileList: TButton;
    bClearLargeText: TButton;
    bClearBroadcastedMessages: TButton;
    bClearPicture: TButton;
    bClearFileList: TButton;
    bSendFile: TButton;
    bLoadPicture: TButton;
    bSendPicture: TButton;
    bSendText: TButton;
    bSendLargeText: TButton;
    bLoadLargeText: TButton;
    edText: TEdit;
    edSendFile: TFileNameEdit;
    imPicture: TImage;
    imScreen: TImage;
    lbBroadcastedFileList: TListBox;
    lbBroadcastedFiles: TLabel;
    lbBroadcastedMessages: TLabel;
    lbClient: TLabel;
    lbSendFile: TLabel;
    lbBroadcasting: TLabel;
    lbServerBroadcast: TLabel;
    lbNotConnected: TLabel;
    lbPicture: TLabel;
    lbRecvFiles: TLabel;
    lbText: TLabel;
    lbLargeText: TLabel;
    lbRecvFileList: TListBox;
    lwInfo: TListView;
    mLargeText: TMemo;
    mBroadcastedMessages: TMemo;
    OD: TOpenDialog;
    OPD: TOpenPictureDialog;
    pnClients: TPanel;
    pgPages: TPageControl;
    sbPicture: TScrollBox;
    sbScreen: TScrollBox;
    tsInformation: TTabSheet;
    tsClient: TTabSheet;
    tsFileB: TTabSheet;
    tsMessage: TTabSheet;
    tsScreen: TTabSheet;
    tsBasicOperations: TTabSheet;
    tsFile: TTabSheet;
    tvPages: TTreeView;
    tsPicture: TTabSheet;
    tsServerBroadcast: TTabSheet;
    tsText: TTabSheet;
    procedure bClearBroadcastedFileListClick(Sender: TObject);
    procedure bClearFileList1Click(Sender: TObject);
    procedure bClearBroadcastedMessagesClick(Sender: TObject);
    procedure bClearLargeTextClick(Sender: TObject);
    procedure bClearFileListClick(Sender: TObject);
    procedure bLoadLargeTextClick(Sender: TObject);
    procedure bClearPictureClick(Sender: TObject);
    procedure bLoadPictureClick(Sender: TObject);
    procedure bSendFileClick(Sender: TObject);
    procedure bSendLargeTextClick(Sender: TObject);
    procedure bSendPictureClick(Sender: TObject);
    procedure bSendTextClick(Sender: TObject);
    procedure edTextChange(Sender: TObject);
    procedure edSendFileChange(Sender: TObject);
    procedure imPicturePictureChanged(Sender: TObject);
    procedure mLargeTextChange(Sender: TObject);
    procedure tvPagesChange(Sender: TObject; Node: TTreeNode);
  private
    FTCPBase: TTCPBase;
    FTCPBaseType: TBaseType;
    FSettings: TSettings;
    FConnection: TConnection;
    procedure DoClick(Sender: TObject);
    procedure PopulateClientInfo;
  public
    procedure InitFrame(ATCPBase: TTCPBase; ATCPBaseType: TBaseType; ASettings: TSettings; AConnection: TConnection);
    procedure RecvMessage(Sender: TObject; AMsg: String; AParams: TStringArray; {%H-}AConnection: TConnection; {%H-}AddLog: TAddLog);
    procedure RecvStream(Sender: TObject; AMsg: String; {%H-}AParams: TStringArray; AMS: TMemoryStream; {%H-}AConnection: TConnection; {%H-}AddLog: TAddLog);
    procedure RecvFile(Sender: TObject; AMsg: String; {%H-}AParams: TStringArray; AFileName: String; {%H-}AConnection: TConnection; {%H-}AddLog: TAddLog);
  public
    property Connection: TConnection read FConnection;
  end;

implementation

uses uTCPCryptoCompression;

{$R *.lfm}

{ TfrClient }

procedure TfrClient.InitFrame(ATCPBase: TTCPBase; ATCPBaseType: TBaseType;
  ASettings: TSettings; AConnection: TConnection);
var
  pm: TPopupMenu;
  miClose: TMenuItem;
begin
  FTCPBase := ATCPBase;
  FTCPBaseType := ATCPBaseType;
  FSettings := ASettings;
  FConnection := AConnection;
  if FTCPBaseType = btServer then
  begin
    pm := TPopupMenu.Create(Self);
    pm.Parent := Self.Parent;
    TTabSheet(Self.Parent).PopupMenu := pm;
    miClose := TMenuItem.Create(pm);
    miClose.Caption := 'Close tab';
    miClose.OnClick := @DoClick;
    pm.Items.Add(miClose);
    PopulateClientInfo;
    pgPages.Visible := True;
    tvPages.Items[tsServerBroadcast.PageIndex].Delete;
  end
  else if FTCPBaseType = btClient then
  begin
    FConnection.FUniqueID := 0;
    tvPages.Items[tsClient.PageIndex].Delete;
    pgPages.Visible := False;
  end;
  pgPages.ShowTabs := False;
  pgPages.ActivePageIndex := -1;
  tvPages.Items[0].Selected := True;
end;

procedure TfrClient.PopulateClientInfo;

  procedure AddItem(const AValue, AData: String);
  begin
    with lwInfo.Items.Add do
    begin
      Caption := AValue;
      Subitems.Add(AData);
    end;
  end;

begin
  AddItem('IP', FConnection.FExtendedInfo.FRemoteIP);
  AddItem('Host', FConnection.FExtendedInfo.FRemoteHost);
  AddItem('User', FConnection.FUser);
  AddItem('UniqueID', IntToStr(FConnection.FUniqueID));
  AddItem('Connected since', FormatDateTime('YYYY.MM.DD hh:mm:ss', FConnection.FDateTime));
end;

procedure TfrClient.DoClick(Sender: TObject);
begin
  TTabSheet(Self.Parent).TabVisible := False;
end;

{$Region 'Irrelevant stuff'}
procedure TfrClient.bLoadLargeTextClick(Sender: TObject);
begin
  if OD.Execute then
    mLargeText.Lines.LoadFromFile(OD.FileName);
end;

procedure TfrClient.bClearLargeTextClick(Sender: TObject);
begin
  mLargeText.Clear;
end;

procedure TfrClient.bLoadPictureClick(Sender: TObject);
var
  Bmp: TBitmap;
  Jpg: TJPEGImage;
  Png: TPortableNetworkGraphic;
begin
  if OPD.Execute then
  begin
    case UpperCase(ExtractFileExt(OPD.FileName)) of
      '.BMP': begin
               Bmp := TBitmap.Create;
               try
                 Bmp.LoadFromFile(OPD.FileName);
                 imPicture.Picture.Bitmap := Bmp;
               finally
                 Bmp.Free;
               end;
             end;
      '.PNG': begin
                Png := TPortableNetworkGraphic.Create;
                try
                  Png.LoadFromFile(OPD.FileName);
                  imPicture.Picture.Graphic := Png;
                finally
                  Png.Free;
                end;
              end;
      '.JPG', '.JPEG':
             begin
               Jpg := TJPEGImage.Create;
               try
                 Jpg.LoadFromFile(OPD.FileName);
                 imPicture.Picture.Graphic := Jpg;
               finally
                 Jpg.Free;
               end;
             end;
    end;
  end;
end;

procedure TfrClient.bClearPictureClick(Sender: TObject);
begin
  imPicture.Picture.Clear;
  imPicture.Width := 100;
  imPicture.Height := 100;
  imPicture.Picture := nil;
end;

procedure TfrClient.bClearFileListClick(Sender: TObject);
begin
  lbRecvFileList.Clear;
end;

procedure TfrClient.bClearFileList1Click(Sender: TObject);
begin
  lbBroadcastedFileList.Clear;
end;

procedure TfrClient.bClearBroadcastedFileListClick(Sender: TObject);
begin
  lbBroadcastedFileList.Clear;
end;

procedure TfrClient.bClearBroadcastedMessagesClick(Sender: TObject);
begin
  mBroadcastedMessages.Clear;
end;

procedure TfrClient.edTextChange(Sender: TObject);
begin
  bSendText.Enabled := Trim(edText.Text) <> '';
end;

procedure TfrClient.edSendFileChange(Sender: TObject);
begin
  bSendFile.Enabled := (Trim(edSendFile.Text) <> '') and (FileExists(edSendFile.Text));
end;

procedure TfrClient.imPicturePictureChanged(Sender: TObject);
begin
  bSendPicture.Enabled := imPicture.Picture.Graphic <> nil;
end;

procedure TfrClient.mLargeTextChange(Sender: TObject);
begin
  bSendLargeText.Enabled := Trim(mLargeText.Text) <> '';
end;

procedure TfrClient.tvPagesChange(Sender: TObject; Node: TTreeNode);
begin
  pgPages.ActivePageIndex := Node.StateIndex;
  pgPages.Invalidate;
end;
{$EndRegion}

procedure TfrClient.bSendTextClick(Sender: TObject);
begin
  FTCPBase.SendMessage('TEXT', [edText.Text], FConnection.FUniqueID);
end;

procedure TfrClient.bSendLargeTextClick(Sender: TObject);
var
  MS: TMemoryStream;
begin
  //no need to free the MS, uTCPSockets will take care of it
  MS := TMemoryStream.Create;
  mLargeText.Lines.SaveToStream(MS);
  MS.Position := 0;
  FTCPBase.SendStream('LARGETEXT', [], MS, FConnection.FUniqueID);
end;

procedure TfrClient.bSendPictureClick(Sender: TObject);
var
  MS: TMemoryStream;
begin
  MS := TMemoryStream.Create;
  imPicture.Picture.Graphic.SaveToStream(MS);
  MS.Position := 0;
  FTCPBase.SendStream('PICTURE', [], MS, FConnection.FUniqueID);
end;

procedure TfrClient.bSendFileClick(Sender: TObject);
begin
  FTCPBase.SendFile('FILE', ['FILE'], edSendFile.FileName, FConnection.FUniqueID);
end;

procedure TfrClient.RecvMessage(Sender: TObject; AMsg: String;
  AParams: TStringArray; AConnection: TConnection; AddLog: TAddLog);
begin
  case AMsg of
    'TEXT':
       begin
         edText.Text := AParams[0];
         pgPages.ActivePage := tsText;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
    'BROADCASTMESSAGE':
       begin
         mBroadcastedMessages.Lines.Add(FormatDateTime('hh:mm:ss', now) + ' : Message: ' + AParams[0] + '(params: ' + AParams[1] + ')');
         pgPages.ActivePage := tsMessage;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
  end;
end;

procedure TfrClient.RecvStream(Sender: TObject; AMsg: String; AParams: TStringArray;
  AMS: TMemoryStream; AConnection: TConnection; AddLog: TAddLog);
var
  JPeg: TJPEGImage;
  MSDecomp: TMemoryStream;
begin
  case AMsg of
    'LARGETEXT':
       begin
         AMS.Position := 0;
         mLargeText.Lines.LoadFromStream(AMS);
         pgPages.ActivePage := tsText;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
    'PICTURE':
       begin
         AMS.Position := 0;
         imPicture.Picture.LoadFromStream(AMS);
         pgPages.ActivePage := tsPicture;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
    'BROADCASTSCREEN':
       begin
         MSDecomp := TMemoryStream.Create;
         try
           AMS.Position := 0;
           DecompressStream(AMS, MSDecomp);
           JPeg := TJPEGImage.Create;
           try
             MSDecomp.Position := 0;
             try
               JPeg.LoadFromStream(MSDecomp);
               imScreen.Picture.Graphic := Jpeg;
             except
             end;
           finally
             JPeg.Free;
           end;
         finally
           MSDecomp.Free;
         end;
         pgPages.ActivePage := tsScreen;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
  end;
end;

procedure TfrClient.RecvFile(Sender: TObject; AMsg: String; AParams: TStringArray;
  AFileName: String; AConnection: TConnection; AddLog: TAddLog);
begin
  case AMsg of
    'FILE':
       begin
         lbRecvFileList.Items.Add(FormatDateTime('hh:mm:ss', now) + ' : ' + AFileName);
         pgPages.ActivePage := tsFile;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
    'BROADCASTFILE':
       begin
         lbBroadcastedFileList.Items.Add(FormatDateTime('hh:mm:ss', now) + ' : ' + AFileName);
         pgPages.ActivePage := tsFileB;
         tvPages.Items[pgPages.ActivePageIndex].Selected := True;
       end;
  end;
end;

end.

