unit uSettings;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Laz2_XMLCfg, LazFileUtils, uTCPSockets;

type

  { TSettings }
  TSettings = class
  private
    FVersion: Integer;
    FFileName: String;
    FAutoStart: Boolean;
    FBindConnection: TConnection;
    FSaveCredentials: Boolean;
    FHideInternalMessages: Boolean;
    FHideErrorMessages: Boolean;
    FDownloadDirectory: String;
  public
    constructor Create;
    destructor Destroy; override;
  public
    procedure Load;
    procedure Save;
  public
    property FileName: string read FFileName write FFileName;
    property AutoStart: Boolean read FAutoStart write FAutoStart;
    property SaveCredentials: Boolean read FSaveCredentials write FSaveCredentials;
    property HideInternalMessages: Boolean read FHideInternalMessages write FHideInternalMessages;
    property HideErrorMessages: Boolean read FHideErrorMessages write FHideErrorMessages;
    property DownloadDirectory: String read FDownloadDirectory write FDownloadDirectory;
    property BindConnection: TConnection read FBindConnection write FBindConnection;
  end;

implementation

uses uTCPCryptoCompression;

const
  InternalVersion = 1;

constructor TSettings.Create;
begin
  FBindConnection := TConnection.Create;
end;

destructor TSettings.Destroy;
begin
  FBindConnection.Free;
  inherited Destroy;
end;

procedure TSettings.Load;
var
  XML: TXMLConfig;
begin
  XML := TXMLConfig.Create(FFileName);
  try
    FVersion := XML.GetValue('Version/Value', 1);
    FAutoStart := XML.GetValue('AutoStart/Value', False);
    FSaveCredentials := XML.GetValue('SaveCredentials/Value', False);
    FHideInternalMessages := XML.GetValue('HideInternalMessages/Value', False);
    FHideErrorMessages := XML.GetValue('HideErrorMessages/Value', False);
    FDownloadDirectory := XML.GetValue('DownloadDirectory/Value', '');
    FBindConnection.FIP := XML.GetValue('BindIP/Value', '0.0.0.0');
    FBindConnection.FPort := XML.GetValue('Port/Value', '40516');
    if FSaveCredentials then
      FBindConnection.FPass := Decrypt(SHA1Hash(FBindConnection.FPort), XML.GetValue('Pass/Value', ''));
  finally
    XML.Free;
  end;
end;

procedure TSettings.Save;
var
  XML: TXMLConfig;
begin
  XML := TXMLConfig.CreateClean(FFileName);
  try
    XML.SetDeleteValue('Version/Value', InternalVersion, 1);
    XML.SetDeleteValue('AutoStart/Value', FAutoStart, False);
    XML.SetDeleteValue('SaveCredentials/Value', FSaveCredentials, False);
    XML.SetDeleteValue('HideInternalMessages/Value', FHideInternalMessages, False);
    XML.SetDeleteValue('DownloadDirectory/Value', FDownloadDirectory, '');
    XML.SetDeleteValue('BindIP/Value', FBindConnection.FIP, '0.0.0.0');
    XML.SetDeleteValue('Port/Value', FBindConnection.FPort, '40516');
    if FSaveCredentials then
      XML.SetDeleteValue('Pass/Value', Encrypt(SHA1Hash(FBindConnection.FPort), FBindConnection.FPass), '');
    XML.Flush;
  finally
    XML.Free;
  end;
end;

end.
